=== modified file '__init__.py'
--- __init__.py	2006-11-06 10:10:55 +0000
+++ __init__.py	2006-11-06 10:35:46 +0000
@@ -21,7 +21,9 @@
 import os
 import time
 
+import bzrlib
 from bzrlib.commands import Command, display_command, register_command
+from bzrlib import errors
 from bzrlib.option import Option
 
 
@@ -38,7 +40,6 @@
 
     @display_command
     def run(self, by_date=False, all=False, dead_only=False, debug_time=False):
-        from bzrlib import errors
         from bzrlib.osutils import format_date
         import bzrlib.repository
 
@@ -91,28 +92,26 @@
 
         ## mark heads as dead or alive
         # mark all heads as dead
-        head_mark = {}
+        self.head_mark = {}
         for head in heads:
-            head_mark[head] = 'dead'
+            self.head_mark[head] = 'dead'
         # give the list of live branches in repository or current branch tip
         if not branch:
             # runs from shared repo root
-            for directory in os.listdir(u'.'):
-                if directory == '.bzr':
-                    continue
-                if not os.path.isdir(os.path.join(directory, '.bzr')):
-                    continue
-                try:
-                    b, path = bzrlib.branch.Branch.open_containing(directory)
-                except errors.NotBranchError:
-                    continue
-                last_revid = b.last_revision()
-                head_mark[last_revid] = 'alive'
-#                self._print_elapsed_time('proceed branch ' + str(b)+':')
+            self._iter_branches_update_marks(u'.')
         else:
             # runs from branch
-            # TODO: implementation
-            raise NotImplementedError('TODO: need to detect is there is shared repo or not')
+            repo_base = repo.bzrdir.transport.base
+            branch_base = branch.bzrdir.transport.base
+            if repo_base == branch_base:
+                # standalone branch
+                last_revid = branch.last_revision()
+                self.head_mark[last_revid] = 'alive'
+            else:
+                # repo
+                from bzrlib.urlutils import (dirname, local_path_from_url)
+                repo_root = local_path_from_url(dirname(repo_base))
+                self._iter_branches_update_marks(repo_root)
         self._print_elapsed_time('make head marks:')
 
         # show time
@@ -121,7 +120,7 @@
         
         for head in heads:
 
-            mark = head_mark[head]
+            mark = self.head_mark[head]
             if not all:
                 if dead_only:
                     if mark != 'dead':
@@ -161,6 +160,23 @@
             print >>to_file, 'No heads found'
             return 1
 
+    def _iter_branches_update_marks(self, root):
+        cwd = os.getcwdu()
+        os.chdir(root)
+        for directory in os.listdir(u'.'):
+            if directory == '.bzr':
+                continue
+            if not os.path.isdir(os.path.join(directory, '.bzr')):
+                continue
+            try:
+                b = bzrlib.branch.Branch.open_containing(directory)[0]
+            except errors.NotBranchError:
+                continue
+            last_revid = b.last_revision()
+            self.head_mark[last_revid] = 'alive'
+#                self._print_elapsed_time('proceed branch ' + str(b)+':')
+        os.chdir(cwd)
+
     def _init_elapsed_time(self, debug_time=False):
         self.debug_time = debug_time
         if debug_time:

